/* **********************************************************************
*Copyright (C) 2000 Elliott Mitchell					*
*All rights reserved.							*
*									*
*$Id: common.c,v 2.5.1.0 2002/06/04 02:03:06 ehem Exp $			*
*************************************************************************
*CS410-Combinatorial Search Project					*
************************************************************************/


#include <stdlib.h>
#include "common.h"
#include "libcolorset.h"


void destroyinstance(struct instance *targ)
{
	struct instance *tmp;

	if(!targ) return;

	free(targ->nodeptr);
	do {
		tmp=targ->islands;
		free(targ);
	} while((targ=tmp));
}


/*The coloring utilities*/

/*int finishcolor(struct instance *wk)
{
	trivialcolor(wk);
	savecolor(wk);
	return(1);
}*/

int trivialcolor(struct instance *inst)
{
	register edge_t e;
	register int color1;
#ifdef DEBUG
	register int color2;
#endif
	register struct node *wk;

	wk=inst->trivialnodes;
	while(wk) {
		color1=0;
#ifdef DEBUG
		color2=0;
		for(e=0; e<wk->ecnt; ++e) if(wk->edges[e]->color==1) color1=1;
else if(wk->edges[e]->color==2) color2=1;

		if(color1&&color2) {
			fprintf(stderr, "Internal error #%08X:%08X\n",
__LINE__, __LINE__);
			exit(65);
		}
#else
		for(e=0; e<wk->ecnt&&!color1; ++e) if(wk->edges[e]->color==1)
color1=1;
#endif
		if(color1) wk->color=2;
		else wk->color=1;

		wk=wk->iptr;
	}
	return(1);
}

int clearcolor(struct instance *inst)
{
	struct node *wk;

	wk=inst->nodeptr;

	while(wk->id) {
		wk->color=0;
		wk=(struct node *)(wk->edges+wk->ecnt);
	}
	return(1);
}


/*These could fairly easily be merged, however this is simpler, KISS and fast*/

static void _ichainsort(struct node **, struct node **, int (*)(const struct
node *, const struct node *), int lev);
static void _schainsort(struct node **, struct node **, int (*)(const struct
node *, const struct node *), int lev);

struct node *ichainsort(struct node *ret, int (*compare)(const struct node *,
const struct node *))
{
	int lev=1;
	struct node *wk, *wk0, *wk1, *save;

	_ichainsort(&ret, &save, compare, lev);

	while(save) {
		wk1=save;
		_ichainsort(&wk1, &save, compare, lev);

		wk0=ret;
		if(compare(wk0, wk1)<=0) {
			wk=wk0;
			wk0=wk->iptr;
		} else {
			wk=wk1;
			wk1=wk->iptr;
		}
		ret=wk;

		while(wk0&&wk1) {
			if(compare(wk0, wk1)<=0) {
				wk->iptr=wk0;
				wk=wk0;
				wk0=wk->iptr;
			} else {
				wk->iptr=wk1;
				wk=wk1;
				wk1=wk->iptr;
			}
		}
		if(wk0) wk->iptr=wk0;
		else wk->iptr=wk1;

		++lev;
	}

	return(ret);
}

struct node *schainsort(struct node *ret, int (*compare)(const struct node *,
const struct node *))
{
	int lev=1;
	struct node *wk, *wk0, *wk1, *save;

	_schainsort(&ret, &save, compare, lev);

	while(save) {
		wk1=save;
		_schainsort(&wk1, &save, compare, lev);

		wk0=ret;
		if(compare(wk0, wk1)<=0) {
			wk=wk0;
			wk0=wk->sptr;
		} else {
			wk=wk1;
			wk1=wk->sptr;
		}
		ret=wk;

		while(wk0&&wk1) {
			if(compare(wk0, wk1)<=0) {
				wk->sptr=wk0;
				wk=wk0;
				wk0=wk->sptr;
			} else {
				wk->sptr=wk1;
				wk=wk1;
				wk1=wk->sptr;
			}
		}
		if(wk0) wk->sptr=wk0;
		else wk->sptr=wk1;

		++lev;
	}

	return(ret);
}


int diffcmp(const struct node *a, const struct node *b)
{
	if(a->diff>b->diff) return(-1);
	else if(a->diff<b->diff) return(1);
	else return(0);
}



void _ichainsort(struct node **wk0, struct node **wk1, int (*compare)(const
struct node *, const struct node *), int lev)
{
	struct node *wk, *wk2, *wk3;
	if(lev<=1) {
		wk=*wk0;
		if(lev==0) {
			*wk1=wk->iptr;
			wk->iptr=NULL;
		} else if(wk->iptr) {
			*wk1=(wk2=wk->iptr)->iptr;
			if(compare(wk, wk->iptr)>0) {
				*wk0=wk2;
				wk2->iptr=wk;
				wk->iptr=NULL;
			} else wk2->iptr=NULL;
		} else *wk1=NULL;
		return;
	}
	wk2=*wk0;
	_ichainsort(wk0, &wk3, compare, lev-1);
	if(!wk3) *wk1=NULL;
	else {
		wk2=*wk0;
		_ichainsort(&wk3, wk1, compare, lev-1);

		if(compare(wk2, wk3)<=0) {
			wk=wk2;
			wk2=wk->iptr;
		} else {
			wk=wk3;
			wk3=wk->iptr;
		}
		*wk0=wk;

		while(wk2&&wk3) {
			if(compare(wk2, wk3)<=0) {
				wk->iptr=wk2;
				wk=wk2;
				wk2=wk->iptr;
			} else {
				wk->iptr=wk3;
				wk=wk3;
				wk3=wk->iptr;
			}
		}
		if(wk2) wk->iptr=wk2;
		else wk->iptr=wk3;
	}

	return;
}

void _schainsort(struct node **wk0, struct node **wk1, int (*compare)(const
struct node *, const struct node *), int lev)
{
	struct node *wk, *wk2, *wk3;
	if(lev<=1) {
		wk=*wk0;
		if(lev==0) {
			*wk1=wk->sptr;
			wk->sptr=NULL;
		} else if(wk->sptr) {
			*wk1=(wk2=wk->sptr)->sptr;
			if(compare(wk, wk->sptr)>0) {
				*wk0=wk2;
				wk2->sptr=wk;
				wk->sptr=NULL;
			} else wk2->sptr=NULL;
		} else *wk1=NULL;
		return;
	}
	wk2=*wk0;
	_schainsort(wk0, &wk3, compare, lev-1);
	if(!wk3) *wk1=NULL;
	else {
		wk2=*wk0;
		_schainsort(&wk3, wk1, compare, lev-1);

		if(compare(wk2, wk3)<=0) {
			wk=wk2;
			wk2=wk->sptr;
		} else {
			wk=wk3;
			wk3=wk->sptr;
		}
		*wk0=wk;

		while(wk2&&wk3) {
			if(compare(wk2, wk3)<=0) {
				wk->sptr=wk2;
				wk=wk2;
				wk2=wk->sptr;
			} else {
				wk->sptr=wk3;
				wk=wk3;
				wk3=wk->sptr;
			}
		}
		if(wk2) wk->sptr=wk2;
		else wk->sptr=wk3;
	}

	return;
}

