/* **********************************************************************
*Copyright (C) 2002 Elliott Mitchell					*
*All rights reserved.							*
*									*
*$Id: common.h,v 3.6 2002/05/12 06:02:29 ehem Exp $			*
*************************************************************************
*CS410-Combinatorial Search Project					*
************************************************************************/


#ifndef __COMMON_H__
#define __COMMON_H__

#include <stdio.h>
#include "types.h"

#ifdef BITS64
#if defined(BITS16) || defined(BITS32)
#error "Only one size level may be specified"
#endif
#warning "Though theoretically supported, 64-bit may exceed resources"
#define BITS 64
#define TYPEMASK 0xFFFFFFFFFFFFFFFF
typedef uint64 uintsz;
typedef int64 intsz;
#else
#ifdef BITS32
#if defined(BITS16)
#error "Only one size level may be specified"
#endif
#define BITS 32
#define TYPEMASK 0xFFFFFFFF
typedef uint32 uintsz;
typedef int32 intsz;
#else
#ifndef BITS16
#define BITS16
#endif
#define BITS 16
#define TYPEMASK 0xFFFF
typedef uint16 uintsz;
typedef int16 intsz;
#endif
#endif


/*LIMITS*/
/*Beyond BITS32 these variables start to do meltdowns. Be aware that each step
*in size doubles the structures; making them much more likely to spill out of
*processor cache and into memory*/
/*This is ruled by the number of nodes*/
#define NODEMAX TYPEMASK
typedef uintsz node_t;
/*This is ruled by the maximum number of edges per node*/
#define EDGEMAX TYPEMASK
typedef uintsz edge_t;

#define COLORMAX TYPEMASK
typedef uintsz color_t;

typedef struct colorset *ColorSet;


struct instance {
	struct instance *islands; /*Pointer to island header or next header*/
	node_t nodecnt; /*How many nodes in this instance/island*/
	struct node *nodeptr; /*Ptr to nodes; For instance is simply mem blk*/
	color_t bestcolors; /*The best we've found so far*/
	struct node *workchain; /*Trivial nodes on main, work on others*/
};

struct node {
	node_t id; /*Slightly important issue*/

	struct node *iptr; /*Island pointer*/
	struct node *sptr; /*Scratch ordering pointer*/

	unsigned trivial:1; /*A useful flag*/
	color_t bestcolor; /*Saving the best if we can't better*/
	color_t color; /*Working color*/

	edge_t diff; /*Difficulty, number of non-trivial neighbors*/

	color_t disc;
	ColorSet colset;

	edge_t ecnt; /*How many edges?*/
	struct node *edges[1]; /*The fun variable size structure*/
};


/*unfortunatly gcc does not allow arrays of bits so we've got to do this*/
struct colorset {
        color_t size;
        unsigned long data[1]; /*The size of this is actually variable*/
};


/**destroy* instance*/
extern void destroyinstance(struct instance *);

/*Load a file and build an instance*/
extern struct instance *doinput(FILE *);

/*Output a colored instance*/
extern int dooutput(FILE *, struct instance *);

/*Simple coloring function, note *destroys* old coloring data!*/
extern int dumbcolor(struct instance *);

/*Full coloring search; *requires* instance data to be fully initialized*/
extern int search(struct instance *);

/*Coloring utilities*/
extern int finishcolor(struct instance *);
extern int savecolor(struct instance *);
extern int trivialcolor(struct instance *);
extern int clearcolor(struct instance *);

#endif

