/* **********************************************************************
*Copyright (C) 2002 Elliott Mitchell					*
*All rights reserved.							*
*									*
*$Id: dumb.c,v 3.7 2002/05/23 22:10:08 ehem Exp $			*
*************************************************************************
*CS410-Combinatorial Search Project					*
************************************************************************/


#define DBGFILE 3

#include "common.h"
#include "libcolorset.h"


#ifndef NODECMP
#ifndef REVERSE
#define NODECMP(a, b) ((a)->diff>=(b)->diff)
#else
#define NODECMP(a, b) ((a)->diff<(b)->diff)
#endif
#endif

static int islandsort(struct instance *);
static int chainsetup(struct instance *);


int dumbcolor(struct instance *inst)
{
	struct instance *instwk;
	register struct node *wk;
	struct node *last;
	register edge_t e;
	register color_t c;
	register ColorSet col;
	color_t maxcol;

	instwk=inst->islands;
	CS_NEW(col);
	while(instwk) {
		chainsetup(instwk);
		wk=instwk->nodeptr;
		instwk->workchain=instwk->nodeptr->iptr; /*Special srch setup*/
		wk->color=1; /*Another tiny bit of srch setup*/

		maxcol=0;
		last=NULL;
		while(wk) {
			CS_CLEAR(col);
			for(e=0; e<wk->ecnt; ++e) CS_SET(col,
wk->edges[e]->bestcolor);

			c=1;
			while(CS_TST(col, c)) ++c;
			wk->bestcolor=c; /*we're run first so we set this*/

			if(c>maxcol) maxcol=c;

			wk->sptr=last; /*Another bit of prep for later*/
			last=wk;

			wk=wk->iptr;
		}

		instwk->bestcolors=maxcol;
		if(maxcol>inst->bestcolors) inst->bestcolors=maxcol;

		instwk=instwk->islands;
	}
	CS_DEL(col);

/*	trivialcolor(inst); **The trivial nodes really /are/ irrelevant*/

	islandsort(inst); /*prep for main searching*/

	return(1);
}


/************************************************************************
*This code is a candidate for common.c. However it is unlikely		*
*searching an island will change it's position on the chain very much,	*
*and it must be done incrementally later (for which the below has high	*
*overhead).								*
************************************************************************/

void _islandsort(struct instance **, struct instance **, int);

int islandsort(struct instance *inst)
{
	struct instance *head, *wk, *wk0, *wk1, *save;
	int lev=1;

	head=inst->islands;

	_islandsort(&head, &save, lev);

	while(save) {
		wk1=save;
		_islandsort(&wk1, &save, lev);

		wk0=head;
		if(wk0->bestcolors>=wk1->bestcolors) {
			wk=wk0;
			wk0=wk->islands;
		} else {
			wk=wk1;
			wk1=wk->islands;
		}
		head=wk;

		while(wk0&&wk1) {
			if(wk0->bestcolors>=wk1->bestcolors) {
				wk->islands=wk0;
				wk=wk0;
				wk0=wk->islands;
			} else {
				wk->islands=wk1;
				wk=wk1;
				wk1=wk->islands;
			}
		}
		if(wk0) wk->islands=wk0;
		else wk->islands=wk1;

		++lev;
	}

	inst->islands=head;
	return(1);
}

void _islandsort(struct instance **wk0, struct instance **wk1, int lev)
{
	struct instance *wk, *wk2, *wk3;
	if(lev<=1) {
		wk=*wk0;
		if(lev==0) {
			*wk1=wk->islands;
			wk->islands=NULL;
		} else if(wk->islands) {
			*wk1=(wk2=wk->islands)->islands;
			if(wk->bestcolors<wk2->bestcolors) {
				*wk0=wk2;
				wk2->islands=wk;
				wk->islands=NULL;
			} else wk2->islands=NULL;
		} else *wk1=NULL;
		return;
	}
	wk2=*wk0;
	_islandsort(wk0, &wk3, lev-1);
	if(!wk3) *wk1=NULL;
	else {
		wk2=*wk0;
		_islandsort(&wk3, wk1, lev-1);

		if(wk2->bestcolors>=wk3->bestcolors) {
			wk=wk2;
			wk2=wk->islands;
		} else {
			wk=wk3;
			wk3=wk->islands;
		}
		*wk0=wk;

		while(wk2&&wk3) {
			if(wk2->bestcolors>=wk3->bestcolors) {
				wk->islands=wk2;
				wk=wk2;
				wk2=wk->islands;
			} else {
				wk->islands=wk3;
				wk=wk3;
				wk3=wk->islands;
			}
		}
		if(wk2) wk->islands=wk2;
		else wk->islands=wk3;
	}

	return;
}



/************************************************************************
*The below code had originally been slated for common.c. However I	*
*realized that because there wasn't much work in place for doing	*
*variable ordering it amounts to a static order and so there isn't any	*
*point of trying to do dynamic.						*
************************************************************************/

static void _ichainsort(struct node **, struct node **, int lev);

int chainsetup(struct instance *inst)
{
	struct node *head, *wk, *wk0, *wk1, *save;
	int lev=1;

	head=inst->nodeptr;

	_ichainsort(&head, &save, lev);

	while(save) {
		wk1=save;
		_ichainsort(&wk1, &save, lev);

		wk0=head;
		if(NODECMP(wk0, wk1)) {
			wk=wk0;
			wk0=wk->iptr;
		} else {
			wk=wk1;
			wk1=wk->iptr;
		}
		head=wk;

		while(wk0&&wk1) {
			if(NODECMP(wk0, wk1)) {
				wk->iptr=wk0;
				wk=wk0;
				wk0=wk->iptr;
			} else {
				wk->iptr=wk1;
				wk=wk1;
				wk1=wk->iptr;
			}
		}
		if(wk0) wk->iptr=wk0;
		else wk->iptr=wk1;

		++lev;
	}

	inst->nodeptr=head;
	return(1);
}

void _ichainsort(struct node **wk0, struct node **wk1, int lev)
{
	struct node *wk, *wk2, *wk3;
	if(lev<=1) {
		wk=*wk0;
		if(lev==0) {
			*wk1=wk->iptr;
			wk->iptr=NULL;
		} else if(wk->iptr) {
			*wk1=(wk2=wk->iptr)->iptr;
			if(!NODECMP(wk, wk2)) {
				*wk0=wk2;
				wk2->iptr=wk;
				wk->iptr=NULL;
			} else wk2->iptr=NULL;
		} else *wk1=NULL;
		return;
	}
	wk2=*wk0;
	_ichainsort(wk0, &wk3, lev-1);
	if(!wk3) *wk1=NULL;
	else {
		wk2=*wk0;
		_ichainsort(&wk3, wk1, lev-1);

		if(NODECMP(wk2, wk3)) {
			wk=wk2;
			wk2=wk->iptr;
		} else {
			wk=wk3;
			wk3=wk->iptr;
		}
		*wk0=wk;

		while(wk2&&wk3) {
			if(NODECMP(wk2, wk3)) {
				wk->iptr=wk2;
				wk=wk2;
				wk2=wk->iptr;
			} else {
				wk->iptr=wk3;
				wk=wk3;
				wk3=wk->iptr;
			}
		}
		if(wk2) wk->iptr=wk2;
		else wk->iptr=wk3;
	}

	return;
}

