/* **********************************************************************
*Copyright (C) 2002 Elliott Mitchell					*
*All rights reserved.							*
*									*
*$Id: testsol.c,v 1.1 2002/05/16 04:32:36 ehem Exp $			*
*************************************************************************
*CS410-Combinatorial Search Project					*
************************************************************************/


#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>


struct instance {
	unsigned long long cnt;
	unsigned long long col[1];
};

struct instance *loadsol(FILE *);
int testinst(const struct instance *, FILE *);


int main(int argc, char **argv, char **envp)
{
	FILE *wk;
	int ret;
	struct instance *inst;

	if(argc!=3) {
		fprintf(stderr, "%s: Usage %s <instance> <solution>\n", *argv,
*argv);
		return(1);
	}

	if(!(wk=fopen(argv[2], "r"))) {
		perror(argv[2]);
		return(1);
	}
	inst=loadsol(wk);
	fclose(wk);

	if(!(wk=fopen(argv[1], "r"))) {
		perror(argv[1]);
		return(1);
	}
	ret=testinst(inst, wk);
	fclose(wk);

	printf("Coloring appears to be valid\n");

	return(ret);
}

struct instance *loadsol(FILE *in)
{
	struct instance *ret;
	unsigned long long colpred;
	unsigned long long node, col;
	char c;
	char buf[1024];

	ret=malloc(sizeof(struct instance)-sizeof(unsigned long long));
	ret->cnt=0;

	while((c=getc(in))=='c') fgets(buf, 1024, in); /*Danger!*/
	if(c!='s') {
		fprintf(stderr, "Missing solution line\n");
		exit(1);
	}
	if(!fscanf(in, " col %llu", &colpred)) {
		fprintf(stderr, "Bad solution line\n");
		exit(1);
	}
	while(!feof(in)) {
		while((c=getc(in))=='c'||isspace(c)) if(c=='c')
fgets(buf, 1024, in); /*Danger!*/
		if(c==EOF) break;
		if(c!='l') {
			fprintf(stderr, "Improper line type \'%c\'\n", c);
			exit(1);
		}

		if(!fscanf(in, " %llu %llu", &node, &col)) {
			fprintf(stderr, "Bad label line\n");
			exit(1);
		}
		if(node>ret->cnt) {
			if(!(ret=realloc(ret, sizeof(struct instance)+
sizeof(unsigned long long)*node))) {
				perror("realloc");
				exit(-1);
			}
			ret->cnt=node;
		}
		ret->col[node]=col;
	}

	return(ret);
}

int testinst(const struct instance *dat, FILE *in)
{
	unsigned long long b, e;
	unsigned long long ncnt, ecnt;
	char c;
	char buf[1024];


	while((c=getc(in))=='c') fgets(buf, 1024, in); /*Danger!*/
	if(c!='p') {
		fprintf(stderr, "Missing problem line\n");
		exit(1);
	}
	if(!fscanf(in, " edge %llu %llu", &ncnt, &ecnt)) {
		fprintf(stderr, "Bad problem line\n");
		exit(1);
	}
	if(ncnt!=dat->cnt) {
		if(ncnt>dat->cnt) {
			fprintf(stderr, "Too few nodes in solution\n");
			exit(1);
		} else fprintf(stderr, "Too many nodes in solution\n");
	}

	while(!feof(in)) {
		while((c=getc(in))=='c'||isspace(c)) if(c=='c')
fgets(buf, 1024, in); /*Danger!*/
		if(c==EOF) break;
		if(c!='e') {
			fprintf(stderr, "Improper line type \'%c\'\n", c);
			exit(1);
		}

		if(!fscanf(in, " %llu %llu", &b, &e)) {
			fprintf(stderr, "Bad edge descriptor\n");
			exit(1);
		}
		if(dat->col[b]==dat->col[e]) {
			fprintf(stderr, "Adjacent nodes %llu and %llu have "
"same color\n", b, e);
		}
        }
	return(1);
}

